use std::collections::HashSet;

use std::cell::RefCell;
use std::fmt::Debug;
use std::fmt::Display;

use super::GlobalNamespace;

#[derive(Clone, Copy)]
pub struct Trait<'a> {
    pub(super) id: usize,
    pub(super) namespace: &'a GlobalNamespace,
}

impl Display for Trait<'_> {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        let name = &self.namespace.traits.borrow()[self.id].name;

        f.write_fmt(format_args!("{name}"))
    }
}

impl Debug for Trait<'_> {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "Trait({self})")
    }
}

pub(super) struct InternalTrait {
    // make resolution easier
    pub(super) types: RefCell<HashSet<usize>>,
    pub(super) name: String,
}
