use std::collections::HashSet;

use std::cell::RefCell;
use std::fmt::Debug;
use std::fmt::Display;

use super::Trait;

use super::GlobalNamespace;

#[derive(Clone, Copy)]
pub struct Type<'a> {
    pub(super) id: usize,
    pub(super) namespace: &'a GlobalNamespace,
}

impl<'a> Type<'a> {
    pub fn add_trait(&self, to_add: &'a Trait) {
        let internal_self = &self.namespace.types.borrow()[self.id];

        internal_self.traits.borrow_mut().insert(to_add.id);
        self.namespace.traits.borrow_mut()[to_add.id]
            .types
            .borrow_mut()
            .insert(self.id);
    }

    pub fn has_trait(&self, to_check: &'a Trait) -> bool {
        self.namespace.types.borrow()[self.id]
            .traits
            .borrow()
            .contains(&to_check.id)
    }
}

impl Display for Type<'_> {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        let name = &self.namespace.types.borrow()[self.id].name;

        f.write_fmt(format_args!("{name}"))
    }
}

impl Debug for Type<'_> {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "Type({self})")
    }
}

pub(super) struct InternalType {
    pub(super) traits: RefCell<HashSet<usize>>,
    pub(super) name: String,
}
